<?php
namespace Dfe\AlphaCommerceHub\W;
/**
 * 2017-11-18
 * «AlphaHPP» → «Paypage Request Reference» → «Response Message»
 * http://developer.alphacommercehub.com.au/docs/alphahpp-#response-message
 */
final class Event extends \Df\PaypalClone\W\Event {
	/**
	 * 2017-11-21
	 * 2017-12-08
	 * It correctly works with all currently supported payment options: bank cards, PayPal, and POLi Payments:
	 * *) Bank cards: https://mage2.pro/tags/alphacommercehub-api-response-success-bank-card
	 * *) "A `SuccessURL` response to a POLi Payments payment": https://mage2.pro/t/4961
	 * *) "A PayPal's `PaymentStatus` API request, and a response to it": https://mage2.pro/t/5120
	 * *) "A PayPal's `CapturePayment` API request, and a response to it": https://mage2.pro/t/5127
	 * @used-by \Dfe\AlphaCommerceHub\Block\Info::prepare()
	 * @return string
	 */
	function currencyName() {return df_currency_name($this->r('Result/Currency'));}

	/**
	 * 2017-11-22
	 * @used-by \Dfe\AlphaCommerceHub\Block\Info::prepare()
	 * @param string|null $k
	 * @param mixed|null $d
	 * @return string|null|array(string => string)
	 */
	function providerRespL($k = null, $d = null) {return dfak(df_last($this->providerResps()), $k, $d);}

	/**
	 * 2017-12-05
	 * Note 1. The type of the current transaction.
	 * Note 2.
	 * "If a bank card transaction was just preauthorized (not captured yet),
	 * then my `SuccessURL` handler should mark it so in Magento (currently it wrongly marks it as captured)":
	 * https://github.com/mage2pro/alphacommercehub/issues/65
	 * Note 3.
	 * "[AlphaCommerceHub] A `SuccessURL` response to a preauthorized (not captured) bank card payment":
	 * https://mage2.pro/t/5062
	 * @override
	 * @see \Df\PaypalClone\W\Event::ttCurrent()
	 * @used-by \Df\Payment\W\Strategy\ConfirmPending::_handle()
	 * @used-by \Df\PaypalClone\W\Nav::id()
	 */
	function ttCurrent() {return !$this->isSuccessful() ? parent::ttCurrent() : (
		/**
		 * 2017-12-08
		 * It correctly works with PayPal and POLi payments,
		 * because an event's data for them does not contain the «MethodResult/AuthCode» property,
		 * and we always use self::T_CAPTURE for them.
		 * *) "A `SuccessURL` response to a POLi Payments payment": https://mage2.pro/t/4961
		 * *) "A PayPal's `PaymentStatus` API request, and a response to it": https://mage2.pro/t/5120
		 * *) "A PayPal's `CapturePayment` API request, and a response to it": https://mage2.pro/t/5127
		 */
		$this->r('MethodResult/AuthCode') ? self::T_AUTHORIZE : self::T_CAPTURE
	);}

	/**
	 * 2017-11-18
	 * AlphaCommerceHub does not uses signatures for an unknown reason.
	 * "How should my extension check whether an AlphaHPP's response message
	 * is sent by AlphaCommerceHub or by a hacker?" https://mage2.pro/t/4806
	 * @override
	 * @see \Df\PaypalClone\W\Event::validate()
	 * @used-by \Df\Payment\W\Handler::handle()
	 */
	function validate() {}

	/**
	 * 2017-11-18 «The APC ID for the payment». An example: «104543502».
	 * 2017-11-22
	 * 1) "Which payment options `SuccessURL` responses contain `PaymentID`, and which do not?":
	 * https://mage2.pro/t/4996
	 * 2) POLi Payments: «The request is invalid because the required parameter `Result/PaymentID`
	 * is absent»: https://github.com/mage2pro/alphacommercehub/issues/55
	 * 3) A `SuccessURL` response to a POLi Payments payment: https://mage2.pro/t/4961
	 * 4) Previously, I had retured `Result/PaymentID` here:
	 * https://github.com/mage2pro/alphacommercehub/blob/0.4.1/W/Event.php#L28-L36
	 * But some AlphaCommerceHub's payment options do not return `Result/PaymentID`.
	 * So from now on, I have just use an autogenerated identifier.
	 * Such identifier serves only as `txn_id` for the current Magento transaction.
	 * If in future I will need a more complex solution, then I can use
	 * `ProviderResps/<last>/ProviderName` . '-' . `ProviderResps/<last>/ProviderReference`:
	 * http://developer.alphacommercehub.com.au/docs/api-integration-guidenov-2017#response-message
	 * @override
	 * @see \Df\PaypalClone\W\Event::k_idE()
	 * @used-by \Df\PaypalClone\W\Event::idE()
	 * @return null
	 */
	protected function k_idE() {return null;}

	/**
	 * 2017-11-18 «The merchants transaction id».
	 * 2017-12-08
	 * It correctly works with all currently supported payment options: bank cards, PayPal, and POLi Payments:
	 * *) Bank cards: https://mage2.pro/tags/alphacommercehub-api-response-success-bank-card
	 * *) "A `SuccessURL` response to a POLi Payments payment": https://mage2.pro/t/4961
	 * *) "A PayPal's `PaymentStatus` API request, and a response to it": https://mage2.pro/t/5120
	 * *) "A PayPal's `CapturePayment` API request, and a response to it": https://mage2.pro/t/5127
	 * @override
	 * @see \Df\Payment\W\Event::k_pid()
	 * @used-by \Df\Payment\W\Event::pid()
	 * @return string
	 */
	protected function k_pid() {return 'Result/MerchantTxnID';}

	/**
	 * 2017-11-18
	 * Note 1.
	 * AlphaCommerceHub does not uses signatures for an unknown reason.
	 * "How should my extension check whether an AlphaHPP's response message
	 * is sent by AlphaCommerceHub or by a hacker?" https://mage2.pro/t/4806
	 * Note 2. This method is never used: @see validate()
	 * @override
	 * @see \Df\PaypalClone\W\Event::k_signature()
	 * @return string
	 */
	protected function k_signature() {return null;}

	/**
	 * 2017-11-18
	 * The code for the result of the event. Refer to the appendix.»
	 * http://developer.alphacommercehub.com.au/docs/alphahpp-#response-codes
	 * 2017-12-08
	 * It correctly works with all currently supported payment options: bank cards, PayPal, and POLi Payments:
	 * *) Bank cards: https://mage2.pro/tags/alphacommercehub-api-response-success-bank-card
	 * *) "A `SuccessURL` response to a POLi Payments payment": https://mage2.pro/t/4961
	 * *) "A PayPal's `PaymentStatus` API request, and a response to it": https://mage2.pro/t/5120
	 * *) "A PayPal's `CapturePayment` API request, and a response to it": https://mage2.pro/t/5127
	 * @override
	 * @see \Df\PaypalClone\W\Event::k_status()
	 * @used-by \Df\PaypalClone\W\Event::status()
	 * @return string|null
	 */
	protected function k_status() {return 'Result/ResponseCode';}

	/**
	 * 2017-11-19 «The description of the result of the event».
	 * 2017-12-08
	 * It correctly works with all currently supported payment options: bank cards, PayPal, and POLi Payments:
	 * *) Bank cards: https://mage2.pro/tags/alphacommercehub-api-response-success-bank-card
	 * *) "A `SuccessURL` response to a POLi Payments payment": https://mage2.pro/t/4961
	 * *) "A PayPal's `PaymentStatus` API request, and a response to it": https://mage2.pro/t/5120
	 * *) "A PayPal's `CapturePayment` API request, and a response to it": https://mage2.pro/t/5127
	 * @override
	 * @see \Df\PaypalClone\W\Event::k_statusT()
	 * @used-by \Df\PaypalClone\W\Event::statusT()
	 * @return string|null
	 */
	protected function k_statusT() {return 'Result/ResponseMessage';}

	/**
	 * 2017-11-19 http://developer.alphacommercehub.com.au/docs/alphahpp-#response-codes
	 * 2017-12-08
	 * It correctly works with all currently supported payment options: bank cards, PayPal, and POLi Payments:
	 * *) Bank cards: https://mage2.pro/tags/alphacommercehub-api-response-success-bank-card
	 * *) "A `SuccessURL` response to a POLi Payments payment": https://mage2.pro/t/4961
	 * *) "A PayPal's `PaymentStatus` API request, and a response to it": https://mage2.pro/t/5120
	 * *) "A PayPal's `CapturePayment` API request, and a response to it": https://mage2.pro/t/5127
	 * @override
	 * @see \Df\PaypalClone\W\Event::statusExpected()
	 * @used-by \Df\PaypalClone\W\Event::isSuccessful()
	 * @return int
	 */
	protected function statusExpected() {return 1000;}

	/**
	 * 2017-11-22
	 * «Component capturing 0:N Provider Resp».
	 * «API Integration Guide(Nov 2017)» → «Supported Integrations» → «Response Message»
	 * http://developer.alphacommercehub.com.au/docs/api-integration-guidenov-2017#response-message
	 * 2017-12-08
	 * It correctly works with all currently supported payment options: bank cards, PayPal, and POLi Payments:
	 * *) Bank cards: https://mage2.pro/tags/alphacommercehub-api-response-success-bank-card
	 * *) "A `SuccessURL` response to a POLi Payments payment": https://mage2.pro/t/4961
	 * *) "A PayPal's `PaymentStatus` API request, and a response to it": https://mage2.pro/t/5120
	 * *) "A PayPal's `CapturePayment` API request, and a response to it": https://mage2.pro/t/5127
	 * @used-by providerRespL()
	 * @return array(array(string => string))
	 */
	private function providerResps() {return $this->r('ProviderResps', []);}
}